#!/bin/bash
# Copyright start
# Copyright (C) 2008 - 2025 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end
##################################### CONSTANTS START #####################################
upgrade_fsr_version=$1
local_download_dir=$2
system_connectors_list=$3

# Remove all trailing slashes using parameter expansion
local_download_dir=$(echo "$local_download_dir" | sed 's:/*$::')

local_download_dir="$local_download_dir/$upgrade_fsr_version"

download_fsr_packages_dir="$local_download_dir/fsr-packages"
download_fsr_connectors_dir="$local_download_dir/fsr-connectors"
fsr_release_version_file="/etc/yum/vars/fsr_release_version"
config_dir="/opt/cyops/configs/fsr-elevate/local_download"
yum_custom_vars_dir="$config_dir/vars"
f_config="local_download.conf"
i_download_packages_status=1
i_download_system_connectors_status=1

# color
GREEN='\033[0;32m'
RED='\033[0;31m'
RESET='\033[0m'

##################################### CONSTANTS END ########################################


##################################### FUNCTIONS START  #####################################
convert_kb_to_gb(){

    local size_in_kb=$1
    size_in_kb=$(echo $size_in_kb | sed 's/k//' )

    # Convert GB to KB
    size_in_gb=$(echo "$size_in_kb" | awk '{printf "%.2f", $1 / (1024 * 1024)}')

    echo "$size_in_gb" 

}

convert_gb_to_kb(){

    local size_in_gb=$1
    size_in_gb=$(echo $size_in_gb | sed 's/G//' )

    # Convert KB to GB
    size_in_kb=$(echo "$size_in_gb" | awk '{printf "%.2f", $1 * (1024 * 1024)}')

    echo "$size_in_kb" 

}

convert_mb_to_kb(){

    local size_in_mb=$1
    size_in_mb=$(echo $size_in_mb | sed 's/M//' )

    # Convert KB to GB
    size_in_kb=$(echo "$size_in_mb" | awk '{printf "%.2f", $1 * 1024}')

    echo "$size_in_kb" 

}

get_value_in_kb(){
    s_size=$1
    if [[ $s_size == *'M'* ]]; then
        s_size=$(convert_mb_to_kb $s_size) 
     # Check if output is in 'GB'
    elif [[ $s_size == *'G'* ]];then
        s_size=$(convert_gb_to_kb $s_size) 
    elif [[ $s_size == *'k'* ]];then
        s_size=$(echo $s_size | sed 's/k//' )
    else
        s_size=$(echo $s_size | sed 's/K//' )     
    fi

    echo "$s_size"
}

# Parameters
# Size in KB
# Returns:
# Size in MB if size in kb is > 1024KB
# Size in GB if size in mb is > 1024MB
# Also appends "GB" or "MB" string
convert_size() {
    i_size=$1
    i_one_mb=1024
    i_one_gb=1048576


    if (( $(echo "$i_size > $i_one_gb" | bc -l) )); then
        i_divider=$i_one_gb
        s_unit_i_size="GB"
    elif (( $(echo "$i_size > $i_one_mb" | bc -l) )); then
        i_divider=$i_one_mb
        s_unit_i_size="MB"
    fi
    i_size="$(echo "scale=2; ($i_size / $i_divider)" | bc | sed 's/\.0*$//g') $s_unit_i_size"
}

check_space_requirement(){
    i_available_size=$1
    i_actual_required_size=$2

    if (( $(echo "$i_actual_required_size > $i_available_size" | bc -l) )); then

        convert_size $i_available_size
        i_available_size=$i_size
        convert_size $i_actual_required_size
        i_actual_required_size=$i_size
        exit_with_msg "${RED}error${RESET}: Insufficient space available on $local_download_dir directory\n       Available space : $i_available_size\n       Required space more than : $i_actual_required_size\n"
    fi
}

exit_with_msg(){
    msg=$1
    printf "%b" "$msg\n"
    exit 1
}

get_fsr_packages_size(){
    echo "Checking FortiSOAR packages update..."
    # get the size of the all the FortiSOAR packages
    cmd_download_packages_size=$(echo no | yum --setopt=varsdir=$yum_custom_vars_dir update --disablerepo=fsr-connectors --exclude=fsr-elevate 2>&1 | awk -F": " '/^Total download size/ {print $2}' | grep -v "Operation aborted." | tail -n 1)

    if [ $? != 0 ];then
        exit_with_msg "${RED}error${RESET}: Failed to get the required packages size"
    fi

    # if cmd_download_packages_size is empty means everything is upto date
    if [ -z "$cmd_download_packages_size" ];then
        echo "Nothing to do. FortiSOAR packages are already up-to-date."
        cmd_download_packages_size=-1
    else
        # remove the spaces
        cmd_download_packages_size=$(echo $cmd_download_packages_size | sed "s/ //g")
        cmd_download_packages_size=$(get_value_in_kb $cmd_download_packages_size)

    fi
    download_packages_size=$cmd_download_packages_size
}

download_fsr_packages(){

    download_dir=$1
    msg=$2
    download_size=$3
    cmd_download=$4

    i_return=1

    if [ -d $download_dir ];then
        chattr -i $download_dir
        rm -rf $download_dir
    fi

    # create the dir
    mkdir -p $download_dir

    yum install createrepo -y

    echo "Downloading $msg"

    echo ""

    $cmd_download

    if [ $? != 0 ];then
        exit_with_msg "${RED}error${RESET}: Failed to download the $msg"
    fi

    cmd_check_downloaded_packages_size=$(du -sh $download_dir | awk '{print $1}')

    if [ $? != 0 ];then
        exit_with_msg "${RED}error${RESET}: Failed to get the size of downloaded $msg"
    fi

    cmd_check_downloaded_packages_size=$(echo $cmd_check_downloaded_packages_size | sed "s/ //g")
    cmd_check_downloaded_packages_size=$(get_value_in_kb $cmd_check_downloaded_packages_size)

    # check if all the required packages are downloaded.
    if (( $(echo "$cmd_check_downloaded_packages_size >= $download_size" | bc -l) )); then

        echo -e "${GREEN}Successfully downloaded the $msg in $download_dir.${RESET}"
        createrepo $download_dir
        # make the folder immutable
        chattr +i $download_dir
        i_return=0
    else
        exit_with_msg "${RED}error${RESET}: Failed to download the $msg"
    fi

    return $i_return
}

get_fsr_system_connector_size(){
    echo "Checking system connectors update..."
    # get the size of the system connectors connectors 
    cmd_download_system_connectors_size=$(echo no | yum --setopt=varsdir=$yum_custom_vars_dir update --disablerepo=* --exclude=fsr-elevate --enablerepo=fsr-connectors $system_connectors_list 2>&1 | awk -F": " '/^Total download size/ {print $2}' | grep -v "Operation aborted." | tail -n 1)

    if [ $? != 0 ];then
        exit_with_msg "${RED}error${RESET}: Failed to get the required system connectors size"
    fi

    # if cmd_download_system_connectors_size is empty , means no system connector ia available for upgrade
    if [ -z "$cmd_download_system_connectors_size" ];then
        echo "Nothing to do. System connectors are already up-to-date."
        cmd_download_system_connectors_size=-1
    else
        cmd_download_system_connectors_size=$(echo $cmd_download_system_connectors_size | sed "s/ //g")
        cmd_download_system_connectors_size=$(get_value_in_kb $cmd_download_system_connectors_size)
    fi

    download_system_connectors_size=$cmd_download_system_connectors_size
}

create_download_conf(){
    
    if [ $i_download_packages_status -eq 0 -o $i_download_system_connectors_status -eq 0 ];then
        # create a config dir 
        rm -rf $config_dir
        mkdir -p $config_dir

        # create a conf file
        touch $config_dir/$f_config
        checksum=$(find $local_download_dir -type f -exec sha256sum {} + | sha256sum)
        echo "[$upgrade_fsr_version]" >> $config_dir/$f_config
        echo "flag_upgrade_from_local = True" >> $config_dir/$f_config
        echo "local_download_dir = $local_download_dir" >> $config_dir/$f_config
        echo "checksum = $checksum" >> $config_dir/$f_config

    else
        echo -e "${RED}error${RESET}: Failed to download the packages"    
    fi
}

remove_file(){
    s_file=$1
    # s_file : file name with absolute path
    if [ -f $s_file ];then
        rm -rf $s_file  
    fi
}

create_file(){
    s_file=$1
    # s_file : file name with absolute path
    if [ ! -f $s_file ];then
        touch $s_file
    fi
}

##################################### FUNCTIONS END  #####################################

######################################## MAIN START ######################################

clear
echo ""
echo "***************************************"
echo "Downloading Fortisoar v$upgrade_fsr_version packages."
echo "***************************************"
echo ""
# get the current FSR version
current_fsr_release_version=$(cat $fsr_release_version_file)

# set the yum custom vars
f_fsr_release_version="$yum_custom_vars_dir/fsr_release_version"
f_product_yum_server="$yum_custom_vars_dir/product_yum_server"

# remove the file if present
remove_file $f_fsr_release_version
remove_file $f_product_yum_server

# create file
mkdir -p $yum_custom_vars_dir
create_file $f_fsr_release_version
create_file $f_product_yum_server
echo "$upgrade_fsr_version" > "$yum_custom_vars_dir/fsr_release_version"
echo "repo.fortisoar.fortinet.com" > "$yum_custom_vars_dir/product_yum_server"

get_fsr_packages_size
get_fsr_system_connector_size

if [ ! -d $local_download_dir ];then
    mkdir -p $local_download_dir
fi

cmd_check_available_space=$(df -h $local_download_dir | awk 'NR==2 {print $4}')

if [ $? != 0 ];then
    exit_with_msg "${RED}error${RESET}: Failed to check the available space in / directory"
fi

cmd_check_available_space=$(echo $cmd_check_available_space | sed "s/ //g")
cmd_check_available_space=$(get_value_in_kb $cmd_check_available_space)

# check if adequate space is available to download packages

if [ "$download_packages_size" != "-1" ] && [ "$download_system_connectors_size" != "-1" ]; then
    download_size=$(echo "$download_packages_size + $download_system_connectors_size" | bc)
elif [ "$download_packages_size" = -1 ]; then
    download_size="$download_system_connectors_size"
elif [ "$download_system_connectors_size" = -1 ]; then
    download_size="$download_packages_size"
else
    echo "Nothing to do. System is already up to date."
fi

check_space_requirement $cmd_check_available_space $download_size 

convert_size $download_size
actual_download_size=$i_size

echo ""
echo "FortiSOAR v$upgrade_fsr_version package size to be downloaded for upgrade: $actual_download_size"
echo "FortiSOAR v$upgrade_fsr_version package download path: $local_download_dir"
echo ""
i_count=0

while [ $i_count -le 2 ];do
    echo "Do you want to proceed with the download (y/n)?"
    read s_user_input

    if [ ! -z $s_user_input ]; then
        case $s_user_input in
            "y"|"Y"|"yes"|"Yes"|"YES")
                echo ""
                echo "Starting the packages download..."
                echo ""
                break
            ;;
            "n"|"N"|"no"|"No"|"NO")
                exit_with_msg "Exiting Fortisoar v$upgrade_fsr_version packages download."
            ;;
            *)
                exit_with_msg "You have provided an invalid input. Re-run the command and provide the valid input."
            ;;    
        esac    
    fi
    i_count=$(expr $i_count + 1)
    if [ $i_count -gt 2 ]; then
        echo -e "${RED}error${RESET}: Maximum retries exceeded"
        echo "Exiting download packages."
        exit_with_msg "Re-run the command."
    fi

done

# download fsr packages
if [ $download_packages_size != -1 ];then
    cmd_download="yum --setopt=varsdir=$yum_custom_vars_dir update --downloadonly --downloaddir=$download_fsr_packages_dir --disablerepo=fsr-connectors --exclude=fsr-elevate -y"
    download_fsr_packages $download_fsr_packages_dir "FortiSOAR v$upgrade_fsr_version Packages" $download_packages_size "$cmd_download"
    i_download_packages_status=$?
fi

# download System connectors
if [ $download_system_connectors_size != -1 ];then
    cmd_download="yum --setopt=varsdir=$yum_custom_vars_dir update --downloadonly --downloaddir=$download_fsr_connectors_dir --disablerepo=* --enablerepo=fsr-connectors $system_connectors_list -y"
    download_fsr_packages $download_fsr_connectors_dir "system connectors" $download_system_connectors_size "$cmd_download"
    i_download_system_connectors_status=$?
fi

echo -e "${GREEN}The packages are now successfully downloaded to your local storage $local_download_dir .${RESET}"

create_download_conf
######################################## MAIN END ######################################

