""" Copyright start
  Copyright (C) 2008 - 2025 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """

import os
import sys
import json
import logging
from constants import PHASE_MAP_DICT,EXECUTE_STATUS_FILE
from framework.utility import messages
from framework.utility.config import Config

class Logger(object):
    @staticmethod
    def get_logger(name):
        logger_config = Config.get_config()
        logger = logging.getLogger(name)
        log_path = logger_config.get('log_path')
        log_file_with_timestamp = os.environ.get('LOG_FILE_WITH_TIMESTAMP', log_path)
        
        status_data = {}
        if os.path.isfile(EXECUTE_STATUS_FILE):
            with open(EXECUTE_STATUS_FILE,'r') as file:
                status_data = json.load(file)
        status_data_phase = status_data.get('phase',None)

        target_upgrade_version = None
        try:
            arg = list(set(sys.argv) & set(['-h','--help','create-task','create-shell-script']))
            phase = list(set(sys.argv) & set(PHASE_MAP_DICT.keys()))
            version_required_command = list(set(sys.argv) & set(['check-readiness','execute-task','execute-phase','execute']))
            # Following if blocks handles version in log file for commands which do not accepts version as cli argument 
            if len(sys.argv) > 0 and (arg and arg[0] in sys.argv):
                from framework.utility.task_utils import TaskUtilities
                target_upgrade_version = TaskUtilities.get_current_version()
            # Following elif blocks handles version in log file for commands which accepts version as cli argument 
            elif len(sys.argv) > 0 and (version_required_command and version_required_command[0] in sys.argv):
                from helpers.utils import Utilities
                from framework.utility.task_utils import TaskUtilities
                current_version = TaskUtilities.get_current_version()
                target_upgrade_version = None
                if len(sys.argv) > 0 and '--target-version' in sys.argv:
                    version_arg_index = sys.argv.index('--target-version')+1
                    target_upgrade_version = sys.argv[version_arg_index]
                    # Check provided version is as per pattern
                    Utilities._check_version_for_pattern(target_upgrade_version)
                    c_version = int(current_version.replace('.',''))
                    t_version = int(target_upgrade_version.replace('.',''))

                    # Version comparison for 'execute-phase' command in 'version_required_command' list
                    if version_required_command[0] == 'execute-phase' and (phase and phase[0] in sys.argv):
                        # post-upgrade phase execution is not allowed for target_version greater than current_version
                        if t_version > c_version and (phase and phase[0] == 'post-upgrade'):
                            # Target version is greater than current version
                            print(messages.POST_UPGRADE_EXECUTION_RESTRICTION.format(target_upgrade_version,current_version))
                            sys.exit()
                        if t_version == c_version:
                            # pre-upgrade phase execution is not allowed for target_version same as current_version
                            if (phase and phase[0] == 'pre-upgrade'):
                                # Target version is same as current version
                                print(messages.LOWER_OR_SAME_VERSION_PROVIDED.format(current_version, target_upgrade_version, "same as"))
                                sys.exit()
                            # If post-upgrade phase is executed and provided version is same as current then set target_version with current_version
                            if (phase and phase[0] == 'post-upgrade'):
                                Utilities._set_target_version(target_upgrade_version)
                        if t_version < c_version:
                            # Target version is lower than current version
                            # For pre-upgrade & post-upgrade phase, providing target_version lower than current_version is not allowed
                            print(messages.LOWER_OR_SAME_VERSION_PROVIDED.format(current_version, target_upgrade_version, "less than"))
                            sys.exit()
                    elif version_required_command[0] == 'execute' and status_data:
                        if t_version == c_version and (not status_data_phase or status_data_phase == 'pre-upgrade'):
                            # Target version is same as current version
                            print(messages.LOWER_OR_SAME_VERSION_PROVIDED.format(current_version, target_upgrade_version, "same as"))
                            sys.exit()
                        else:
                            Utilities._set_target_version(target_upgrade_version)
                    # Version comparison for other commands in 'version_required_command' list
                    else:
                        if t_version == c_version:
                            # If task is from post-upgrade phase and provided version is same as current then set target_version with current_version
                            if (version_required_command and version_required_command[0] == 'execute-task') and (phase and phase[0] == 'post-upgrade') and target_upgrade_version == current_version:
                                Utilities._set_target_version(target_upgrade_version)
                            else:
                                # Target version is same as current version
                                print(messages.LOWER_OR_SAME_VERSION_PROVIDED.format(current_version, target_upgrade_version, "same as"))
                                sys.exit()
                        if t_version < c_version:
                            # Target version is lower than current version
                            print(messages.LOWER_OR_SAME_VERSION_PROVIDED.format(current_version, target_upgrade_version, "less than"))
                            sys.exit()
            else:
                from framework.utility.task_utils import TaskUtilities
                target_upgrade_version = TaskUtilities.get_target_version()
        except IndexError as indexErr:
            print(messages.VERSION_ARG_NOT_PROVIDED)
            sys.exit()
        additional_format = {"target_upgrade_version": target_upgrade_version}

        if not logger.hasHandlers():
            logger.setLevel(logger_config.get('log_level'))
            log_file_path = log_file_with_timestamp
            if not os.path.exists(log_file_path) and (arg and arg[0] not in sys.argv) and ('--download-packages' not in sys.argv):
                open(log_file_path, 'x')
            logger_handler = logging.FileHandler(log_file_path)
            logger_handler.setLevel(logging.DEBUG)

            logger_formatter = logging.Formatter(
                '%(asctime)s [%(target_upgrade_version)s] %(levelname)s %(module)s: %(funcName)s(): %(lineno)d: %(message)s',
                datefmt='%Y-%m-%d,%H:%M:%S'
            )
            logger_handler.setFormatter(logger_formatter)
            logger.handlers = []
            logger.addHandler(logger_handler)

        logger = logging.LoggerAdapter(logger, additional_format)
        return logger
